# Copyright 2024 Agnostiq Inc.

import time
from enum import Enum
from typing import TYPE_CHECKING, Union

if TYPE_CHECKING:
    from covalent_cloud.function_serve.models import Deployment

__all__ = [
    "DEPLOY_ELECTRON_PREFIX",
    "SupportedMethods",
    "ServiceStatus",
    "rename",
    "wait_for_deployment_to_be_active",
]


DEPLOY_ELECTRON_PREFIX = "#__deploy_electron__#"

# 120 retries * 30 seconds = 60 minutes
ACTIVE_DEPLOYMENT_RETRIES = 120
ACTIVE_DEPLOYMENT_POLL_INTERVAL = 30


class SupportedMethods(str, Enum):
    GET = "GET"
    POST = "POST"
    PUT = "PUT"
    DELETE = "DELETE"
    PATCH = "PATCH"


class ServiceStatus(str, Enum):
    """Possible statuses for a function service."""

    NEW_OBJECT = "NEW_OBJECT"
    CREATING = "CREATING"
    ACTIVE = "ACTIVE"
    INACTIVE = "INACTIVE"
    ERROR = "ERROR"


class ServeAssetType(str, Enum):
    """Possible types for the ServeAsset `type` field."""

    ASSET = "Asset"
    JSON = "JSON"


def rename(name):
    def decorator(fn):
        fn.__name__ = name
        return fn

    return decorator


def wait_for_deployment_to_be_active(
    deployment: "Deployment", verbose=False
) -> Union[None, "Deployment"]:
    retries_done = 0
    while (
        retries_done < ACTIVE_DEPLOYMENT_RETRIES
        and deployment.status not in [
            ServiceStatus.ERROR,
            ServiceStatus.ACTIVE,
            ServiceStatus.INACTIVE,
        ]
    ):
        if verbose:
            print("Deployment info is: ")
            print(deployment)

        time.sleep(ACTIVE_DEPLOYMENT_POLL_INTERVAL)

        deployment.reload()
        retries_done += 1

    if deployment.status in [
        ServiceStatus.ACTIVE,
        ServiceStatus.ERROR,
    ]:
        return deployment

    if deployment.status == ServiceStatus.INACTIVE:
        raise RuntimeError("Deployment is inactive")

    raise RuntimeError(
        f"Timed out after {ACTIVE_DEPLOYMENT_RETRIES * ACTIVE_DEPLOYMENT_POLL_INTERVAL / 60} "
        "minutes while waiting for the deployment to become active"
    )



# Copyright 2024 Agnostiq Inc.

import typing

from covalent_cloud.function_serve.assets import AssetsMediator
from covalent_cloud.function_serve.common import wait_for_deployment_to_be_active
from covalent_cloud.function_serve.models import Deployment
from covalent_cloud.service_account_interface.client import get_deployment_client
from covalent_cloud.shared.schemas.volume import Volume
from typing import Union

if typing.TYPE_CHECKING:
    from covalent_cloud.function_serve.service_class import FunctionService

__all__ = [
    "deploy",
    "get_deployment",
]


def deploy(function_service: "FunctionService", volume: Volume = None) -> Deployment:
    """
    Deploy a function service to the cloud.
    """

    def deploy_wrapper(*args, **kwargs):

        if volume is not None:
            # Override the volume for the function service
            function_service.volume = volume

        fn_service_model = function_service.get_model(*args, **kwargs)

        assets_mediator = AssetsMediator()
        fn_service_model = assets_mediator.hydrate_assets_from_model(fn_service_model)

        assets_mediator.upload_all()

        dumped_model = fn_service_model.model_dump()

        deployment_client = get_deployment_client()
        response = deployment_client.post(
            "/functions",
            request_options={
                "json": dumped_model,
            },
        )

        deployment = Deployment.from_function_record(response.json())

        # Attach route methods for ease of use
        deployment.attach_route_methods()

        return deployment

    return deploy_wrapper


def get_deployment(function_id: Union[str, Deployment], wait=False) -> Deployment:
    """
    Get the deployment info for a function service.
    """

    if isinstance(function_id, Deployment):
        function_id = function_id.function_id

    deployment_client = get_deployment_client()
    response = deployment_client.get(f"/functions/{function_id}")
    deployment = Deployment.from_function_record(response.json())

    # Attach route methods for ease of use
    deployment.attach_route_methods()

    if wait:
        return wait_for_deployment_to_be_active(deployment)
    return deployment
